/*
 * new_comm.c
 *
 *  Created on: Jan 2, 2017
 *      Author: dawehr
 */
#include "new_comm.h"

#define INTC		XScuGic
#define COMM_UART_DEVICE_ID		XPAR_PS7_UART_0_DEVICE_ID
#define COMM_INTC_DEVICE_ID		XPAR_SCUGIC_SINGLE_DEVICE_ID
#define COMM_UART_INT_IRQ_ID		XPAR_PS7_UART_0_INTR//XPAR_XUARTPS_0_INTR

// Declaration of interrupt handler
void Handler(void *CallBackRef, u32 Event, unsigned int EventData);

int initUartComms() {
	XUartPs* uartInstPtr = uart0_init(COMM_UART_DEVICE_ID, 921600);
	// Initialize UART0 (Bluetooth/WiFi)
	if(!uartInstPtr) {
		return -1;
	}

	uart0_clearFIFOs();

	if (!uart0_int_init(COMM_UART_DEVICE_ID)) {
		return -1;
	}


	/*
	 * Setup the handlers for the UART that will be called from the
	 * interrupt context when data has been sent and received, specify
	 * a pointer to the UART driver instance as the callback reference
	 * so the handlers are able to access the instance data
	 */
	XUartPs_SetHandler(uartInstPtr, (XUartPs_Handler)Handler, uartInstPtr);

	u32 IntrMask = XUARTPS_IXR_RXFULL | XUARTPS_IXR_RXOVR;

	XUartPs_SetInterruptMask(uartInstPtr, IntrMask);


	/*
	 * Set the receiver timeout. If it is not set, and the last few bytes
	 * of data do not trigger the over-water or full interrupt, the bytes
	 * will not be received. By default it is disabled.
	 * Timeout duration = RecvTimeout x 4 x Bit Period. 0 disables the
	 *		timeout function.
	 *
	 * The setting of 8 will timeout after 8 x 4 = 32 character times.
	 * Increase the time out value if baud rate is high, decrease it if
	 * baud rate is low.
	 */
	XUartPs_SetRecvTimeout(uartInstPtr, 8);

	// Second argument is the number of bytes to trigger an interrupt at
	XUartPs_SetFifoThreshold(uartInstPtr, 12);


	return 0;
}

/**************************************************************************/
/**
*
* This function is the handler which performs processing to handle data events
* from the device.  It is called from an interrupt context. so the amount of
* processing should be minimal.
*
* This handler provides an example of how to handle data for the device and
* is application specific.
*
* @param	CallBackRef contains a callback reference from the driver,
*		in this case it is the instance pointer for the XUartPs driver.
* @param	Event contains the specific kind of event that has occurred.
* @param	EventData contains the number of bytes sent or received for sent
*		and receive events.
*
* @return	None.
*
* @note		None.
*
***************************************************************************/
void Handler(void *CallBackRef, u32 Event, unsigned int EventData)
{
	/* data has been received */
	if (Event == XUARTPS_EVENT_RECV_DATA || Event == XUARTPS_EVENT_RECV_TOUT) {
		MIO7_led_on();
	}
}

